package cn.skcks.wx.official.orm;

import com.mybatisflex.codegen.Generator;
import com.mybatisflex.codegen.config.EntityConfig;
import com.mybatisflex.codegen.config.GlobalConfig;
import com.mybatisflex.core.FlexGlobalConfig;
import com.mybatisflex.core.datasource.FlexDataSource;
import lombok.SneakyThrows;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.WebApplicationType;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.builder.SpringApplicationBuilder;

import javax.sql.DataSource;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Map;

@SpringBootApplication
public class OrmGenerator {
    public static void main(String[] args) {
        SpringApplication springApplication = new SpringApplicationBuilder(OrmGenerator.class)
                .profiles("generator")
                .web(WebApplicationType.NONE)
                .build();
        springApplication.run(args);
        generate();
    }

    @SneakyThrows
    private static String getGitUserName() {
        InputStream inputStream = Runtime.getRuntime().exec("git config user.name").getInputStream();
        BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream));
        return reader.readLine();
    }

    private static void generate() {
        FlexDataSource flexDataSource = FlexGlobalConfig.getDefaultConfig().getDataSource();
        Map<String, DataSource> dataSourceMap = flexDataSource.getDataSourceMap();

        String mysql = "mysql";
        generateByDataSource(dataSourceMap, mysql);

    }

    private static void generateByDataSource(Map<String, DataSource> dataSourceMap, String dataSourceName) {
        DataSource dataSource = dataSourceMap.get(dataSourceName);
        //创建配置内容
        GlobalConfig globalConfig = new GlobalConfig();
        globalConfig.setBasePackage("cn.skcks.wx.official.orm");
        globalConfig.setSourceDir(System.getProperty("user.dir") + "/wx-official-orm/src/main/java");
        globalConfig.setAuthor(getGitUserName());

        globalConfig.setEntityDataSource(dataSourceName);
        globalConfig.setEntityGenerateEnable(true);
        globalConfig.setEntityWithSwagger(true);
        globalConfig.setEntityWithLombok(true);
        globalConfig.setEntityGenerateEnable(true);
        globalConfig.setWithActiveRecord(true);
        //设置生成 mapper
        globalConfig.setMapperGenerateEnable(true);
        //设置项目的JDK版本，项目的JDK为14及以上时建议设置该项，小于14则可以不设置
        globalConfig.setEntityJdkVersion(17);
        // 是否覆盖已存在的文件
        globalConfig.setEntityOverwriteEnable(true);

        // 不生成的表
        globalConfig.setUnGenerateTable("DATABASECHANGELOG", "DATABASECHANGELOGLOCK");

        globalConfig.getEntityConfig().setSwaggerVersion(EntityConfig.SwaggerVersion.DOC);
        Generator generator = new Generator(dataSource, globalConfig);
        generator.generate();
    }
}
