package cn.skcks.wx.official.service;

import cn.skcks.wx.official.orm.entity.WxOfficialAccount;
import cn.skcks.wx.official.orm.mapper.WxOfficialAccountMapper;
import com.mybatisflex.core.query.QueryChain;
import jakarta.annotation.PostConstruct;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.api.impl.WxMpServiceImpl;
import me.chanjar.weixin.mp.bean.message.WxMpXmlMessage;
import me.chanjar.weixin.mp.config.WxMpConfigStorage;
import me.chanjar.weixin.mp.config.impl.WxMpDefaultConfigImpl;
import org.apache.commons.lang3.StringUtils;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.concurrent.ConcurrentHashMap;

@Order
@Slf4j
@Service
@RequiredArgsConstructor
public class WxMpGenericService {
    @SuppressWarnings("unused")
    private final WxOfficialAccountMapper wxOfficialAccountMapper;
    private final ConcurrentHashMap<String, WxMpService> wxMpServiceMap = new ConcurrentHashMap<>();

    @PostConstruct
    public void init(){
        QueryChain.of(WxOfficialAccount.class)
                .where(WxOfficialAccount::getEnabled).eq(true)
                .list().forEach(account -> {
            WxMpDefaultConfigImpl config = new WxMpDefaultConfigImpl();
            config.setAppId(account.getAppId());
            config.setSecret(account.getSecret());
            config.setAesKey(account.getAesKey());
            config.setToken(account.getToken());
            WxMpServiceImpl wxMpService = new WxMpServiceImpl();
            wxMpService.setWxMpConfigStorage(config);
            wxMpServiceMap.put(account.getAppId(), wxMpService);
        });
    }

    public void auth(String appId, HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException  {
        log.debug("appId: {}", appId);
        WxMpService wxMpService = wxMpServiceMap.get(appId);
        if(wxMpService == null){
            writeResponse(response, "未找到对应公众号配置");
            return;
        }

        String signature = request.getParameter("signature");
        String nonce = request.getParameter("nonce");
        String timestamp = request.getParameter("timestamp");

        if (!wxMpService.checkSignature(timestamp, nonce, signature)) {
            log.error("非法请求，签名验证失败");
            // 消息签名不正确，说明不是公众平台发过来的消息
            writeResponse(response, "非法请求");
            return;
        }

        String echostr = request.getParameter("echostr");
        if (StringUtils.isNotBlank(echostr)) {
            log.info("微信公众号平台验证请求 echostr: {}", echostr);
            // 说明是一个仅仅用来验证的请求，回显 echostr
            writeResponse(response, echostr);
            return;
        }

        String encryptType = StringUtils.isBlank(request.getParameter("encrypt_type")) ?
                "raw" :
                request.getParameter("encrypt_type");

        if ("raw".equals(encryptType)) {
            // 明文传输的消息
            WxMpXmlMessage inMessage = WxMpXmlMessage.fromXml(request.getInputStream());
            log.info("收到明文消息，内容：{}", inMessage.toString());
            // WxMpXmlOutMessage outMessage = wxMpMessageRouter.route(inMessage);
            // if(outMessage == null) {
                //为null，说明路由配置有问题，需要注意
            writeResponse(response, "");
                // response.getWriter().write("");
            // }
            // response.getWriter().write(outMessage.toXml());
            return;
        }

        if ("aes".equals(encryptType)) {
            // 是aes加密的消息
            String msgSignature = request.getParameter("msg_signature");
            WxMpConfigStorage wxMpConfigStorage = wxMpService.getWxMpConfigStorage();
            WxMpXmlMessage inMessage = WxMpXmlMessage.fromEncryptedXml(request.getInputStream(), wxMpConfigStorage, timestamp, nonce, msgSignature);
            log.info("收到加密消息，内容：{}", inMessage.toString());
            // WxMpXmlOutMessage outMessage = wxMpMessageRouter.route(inMessage);
            // if(outMessage == null) {
                //为null，说明路由配置有问题，需要注意
                // response.getWriter().write("");
            writeResponse(response, "");
            // }
            // response.getWriter().write(outMessage.toEncryptedXml(wxMpConfigStorage));
            return;
        }

        response.getWriter().println("不可识别的加密类型");
    }

    @SneakyThrows
    private void writeResponse(HttpServletResponse response,String content) {
        response.setContentType("text/html;charset=utf-8");
        response.setStatus(HttpServletResponse.SC_OK);
        response.getWriter().write(content);
    }
}
